Profile

Struct Profile 

pub struct Profile(/* private fields */);
A configuration profile: effectively a case-insensitive string.

See the top-level docs for details.

Implementations

Source

impl Profile

pub const Default: Profile

The default profile: "default".

pub const Global: Profile

The global profile: "global".

pub fn new(name: &str) -> Profile

Constructs a profile with the name name.

Example
use figment::Profile;

let profile = Profile::new("staging");
assert_eq!(profile, "staging");
assert_eq!(profile, "STAGING");
pub const fn const_new(name: &'static str) -> Profile

A const to construct a profile with the name name.

Example
use figment::Profile;

const STAGING: Profile = Profile::const_new("staging");

assert_eq!(STAGING, "staging");
assert_eq!(STAGING, "STAGING");
pub fn from_env(key: &str) -> Option<Self>

Constructs a profile from the value of the environment variable with name key, if one is present. The search for key is case-insensitive.

Example
use figment::{Profile, Jail};

Jail::expect_with(|jail| {
    jail.set_env("MY_PROFILE", "secret");

    assert_eq!(Profile::from_env("MY_PROFILE"), Some("secret".into()));
    assert_eq!(Profile::from_env("MY_PROFILE"), Some("secret".into()));
    assert_eq!(Profile::from_env("MY_profile"), Some("secret".into()));
    assert_eq!(Profile::from_env("other_profile"), None);
    Ok(())
});
pub fn from_env_or<P: Into<Profile>>(var: &str, default: P) -> Self

Constructs a profile from the value of the environment variable with name var, if one is present, or default if one is not. The search for var is case-insensitive.

Example
use figment::{Profile, Jail};

Jail::expect_with(|jail| {
    jail.set_env("MY_PROFILE", "secret");

    assert_eq!(Profile::from_env_or("MY_PROFILE", "default"), "secret");
    assert_eq!(Profile::from_env_or("MY_profile", "default"), "secret");
    assert_eq!(Profile::from_env_or("other_prof", "default"), "default");
    Ok(())
});
pub fn as_str(&self) -> &UncasedStr

Converts self into an &UncasedStr.

Example
use figment::Profile;

let profile = Profile::new("static");
let string = profile.as_str();
pub fn starts_with(&self, prefix: &str) -> bool

Returns true iff self case-insensitively starts with prefix.

Example
use figment::Profile;

let profile = Profile::new("static");
assert!(profile.starts_with("STAT"));
assert!(profile.starts_with("stat"));
assert!(profile.starts_with("static"));
pub fn is_custom(&self) -> bool

Returns true iff self is neither “default” nor “global”.

Example
use figment::Profile;

let profile = Profile::new("static");
assert!(profile.is_custom());

assert!(!Profile::Default.is_custom());
assert!(!Profile::Global.is_custom());
pub fn collect(&self, dict: Dict) -> Map<Profile, Dict>

Creates a new map with a single key of *self and a value of dict.

Example
use figment::{Profile, util::map};

let profile = Profile::new("static");
let map = profile.collect(map!["hi".into() => 123.into()]);

Methods from Deref<Target = UncasedStr>

pub fn as_str(&self) -> &str

Returns self as an &str.

Example
use uncased::UncasedStr;

let uncased_str = UncasedStr::new("Hello!");
assert_eq!(uncased_str.as_str(), "Hello!");
assert_ne!(uncased_str.as_str(), "hELLo!");
pub fn len(&self) -> usize

Returns the length, in bytes, of self.

Example
use uncased::UncasedStr;

let uncased_str = UncasedStr::new("Hello!");
assert_eq!(uncased_str.len(), 6);
pub fn is_empty(&self) -> bool

Returns true if self has a length of zero bytes.

Examples
use uncased::UncasedStr;

let s = UncasedStr::new("");
assert!(s.is_empty());

let s = UncasedStr::new("not empty");
assert!(!s.is_empty());
pub fn starts_with(&self, string: &str) -> bool

Returns true if self starts with any casing of the string string; otherwise, returns false.

Example
use uncased::UncasedStr;

let uncased_str = UncasedStr::new("MoOO");
assert!(uncased_str.starts_with("moo"));
assert!(uncased_str.starts_with("MOO"));
assert!(uncased_str.starts_with("MOOO"));
assert!(!uncased_str.starts_with("boo"));

let uncased_str = UncasedStr::new("Bèe");
assert!(!uncased_str.starts_with("Be"));
assert!(uncased_str.starts_with("Bè"));
assert!(uncased_str.starts_with("Bè"));
assert!(uncased_str.starts_with("bèe"));
assert!(uncased_str.starts_with("BèE"));