pub enum Kind {
Message(String),
InvalidType(Actual, String),
InvalidValue(Actual, String),
InvalidLength(usize, String),
UnknownVariant(String, &'static [&'static str]),
UnknownField(String, &'static [&'static str]),
MissingField(Cow<'static, str>),
DuplicateField(&'static str),
ISizeOutOfRange(isize),
USizeOutOfRange(usize),
Unsupported(Actual),
UnsupportedKey(Actual, Cow<'static, str>),
}Expand description
An error kind, encapsulating serde’s serde::de::Error.
Variants
Message(String)
A custom error message.
InvalidType(Actual, String)
An invalid type: (actual, expected). See
serde::de::Error::invalid_type().
InvalidValue(Actual, String)
An invalid value: (actual, expected). See
serde::de::Error::invalid_value().
InvalidLength(usize, String)
Too many or too few items: (actual, expected). See
serde::de::Error::invalid_length().
UnknownVariant(String, &'static [&'static str])
A variant with an unrecognized name: (actual, expected). See
serde::de::Error::unknown_variant().
UnknownField(String, &'static [&'static str])
A field with an unrecognized name: (actual, expected). See
serde::de::Error::unknown_field().
MissingField(Cow<'static, str>)
A field was missing: (name). See serde::de::Error::missing_field().
DuplicateField(&'static str)
A field appeared more than once: (name). See
serde::de::Error::duplicate_field().
ISizeOutOfRange(isize)
The isize was not in range of any known sized signed integer.
USizeOutOfRange(usize)
The usize was not in range of any known sized unsigned integer.
Unsupported(Actual)
The serializer or deserializer does not support the Actual type.
UnsupportedKey(Actual, Cow<'static, str>)
The type .0 cannot be used for keys, need a .1.